@@ -1354,20 +1354,34 @@ SCM_DEFINE (scm_uniform_vector_read_x, "uniform-vector-read!", 1, 3, 0,
 	    "to the value returned by @code{(current-input-port)}.")
 #define FUNC_NAME s_scm_uniform_vector_read_x
 {
-  size_t width;
+  SCM result;
+  size_t c_width, c_start, c_end;
+
   SCM_VALIDATE_BYTEVECTOR (SCM_ARG1, uvec);
 
-  scm_c_issue_deprecation_warning 
+  scm_c_issue_deprecation_warning
     ("`uniform-vector-read!' is deprecated. Use `get-bytevector-n!' from\n"
      "`(rnrs io ports)' instead.");
 
-  width = scm_to_size_t (scm_uniform_vector_element_size (uvec));
+  if (SCM_UNBNDP (port_or_fd))
+    port_or_fd = scm_current_input_port ();
 
-  return scm_get_bytevector_n_x (port_or_fd, uvec,
-                                 scm_from_size_t (scm_to_size_t (start)*width),
-                                 scm_from_size_t ((scm_to_size_t (end)
-                                                   - scm_to_size_t (start))
-                                                  * width));
+  c_width = scm_to_size_t (scm_uniform_vector_element_size (uvec));
+
+  c_start = SCM_UNBNDP (start) ? 0 : scm_to_size_t (start);
+  c_start *= c_width;
+
+  c_end = SCM_UNBNDP (end) ? SCM_BYTEVECTOR_LENGTH (uvec) : scm_to_size_t (end);
+  c_end *= c_width;
+
+  result = scm_get_bytevector_n_x (port_or_fd, uvec,
+				   scm_from_size_t (c_start),
+				   scm_from_size_t (c_end - c_start));
+
+  if (SCM_EOF_OBJECT_P (result))
+    result = SCM_INUM0;
+
+  return result;
 }
 #undef FUNC_NAME
 
@@ -1391,21 +1405,30 @@ SCM_DEFINE (scm_uniform_vector_write, "uniform-vector-write", 1, 3, 0,
 	    "@code{(current-output-port)}.")
 #define FUNC_NAME s_scm_uniform_vector_write
 {
-  size_t width;
-  SCM_VALIDATE_BYTEVECTOR (SCM_ARG1, uvec);
-  port_or_fd = SCM_COERCE_OUTPORT (port_or_fd);
+  size_t c_width, c_start, c_end;
 
-  scm_c_issue_deprecation_warning 
+  SCM_VALIDATE_BYTEVECTOR (SCM_ARG1, uvec);
+
+  scm_c_issue_deprecation_warning
     ("`uniform-vector-write' is deprecated. Use `put-bytevector' from\n"
      "`(rnrs io ports)' instead.");
 
-  width = scm_to_size_t (scm_uniform_vector_element_size (uvec));
+  if (SCM_UNBNDP (port_or_fd))
+    port_or_fd = scm_current_output_port ();
+
+  port_or_fd = SCM_COERCE_OUTPORT (port_or_fd);
+
+  c_width = scm_to_size_t (scm_uniform_vector_element_size (uvec));
+
+  c_start = SCM_UNBNDP (start) ? 0 : scm_to_size_t (start);
+  c_start *= c_width;
+
+  c_end = SCM_UNBNDP (end) ? SCM_BYTEVECTOR_LENGTH (uvec) : scm_to_size_t (end);
+  c_end *= c_width;
 
   return scm_put_bytevector (port_or_fd, uvec,
-                             scm_from_size_t (scm_to_size_t (start)*width),
-                             scm_from_size_t ((scm_to_size_t (end)
-                                               - scm_to_size_t (start))
-                                              * width));
+                             scm_from_size_t (c_start),
+                             scm_from_size_t (c_end - c_start));
 }
 #undef FUNC_NAME
 
@@ -1792,6 +1815,56 @@ scm_i_subr_p (SCM x)
   return SCM_PRIMITIVE_P (x);
 }
 
+
+
+SCM
+scm_internal_lazy_catch (SCM tag, scm_t_catch_body body, void *body_data, scm_t_catch_handler handler, void *handler_data)
+{
+  scm_c_issue_deprecation_warning
+    ("`scm_internal_lazy_catch' is no longer supported. Instead this call will\n"
+     "dispatch to `scm_c_with_throw_handler'. Your handler will be invoked from\n"
+     "within the dynamic context of the corresponding `throw'.\n"
+     "\nTHIS COULD CHANGE YOUR PROGRAM'S BEHAVIOR.\n\n"
+     "Please modify your program to use `scm_c_with_throw_handler' directly,\n"
+     "and adapt it (if necessary) to expect to be within the dynamic context\n"
+     "of the throw.");
+  return scm_c_with_throw_handler (tag, body, body_data, handler, handler_data, 0);
+}
+
+SCM_DEFINE (scm_lazy_catch, "lazy-catch", 3, 0, 0,
+	    (SCM key, SCM thunk, SCM handler),
+	    "This behaves exactly like @code{catch}, except that it does\n"
+	    "not unwind the stack before invoking @var{handler}.\n"
+	    "If the @var{handler} procedure returns normally, Guile\n"
+	    "rethrows the same exception again to the next innermost catch,\n"
+	    "lazy-catch or throw handler.  If the @var{handler} exits\n"
+	    "non-locally, that exit determines the continuation.")
+#define FUNC_NAME s_scm_lazy_catch
+{
+  struct scm_body_thunk_data c;
+
+  SCM_ASSERT (scm_is_symbol (key) || scm_is_eq (key, SCM_BOOL_T),
+	      key, SCM_ARG1, FUNC_NAME);
+
+  c.tag = key;
+  c.body_proc = thunk;
+
+  scm_c_issue_deprecation_warning
+    ("`lazy-catch' is no longer supported. Instead this call will dispatch\n"
+     "to `with-throw-handler'. Your handler will be invoked from within the\n"
+     "dynamic context of the corresponding `throw'.\n"
+     "\nTHIS COULD CHANGE YOUR PROGRAM'S BEHAVIOR.\n\n"
+     "Please modify your program to use `with-throw-handler' directly, and\n"
+     "adapt it (if necessary) to expect to be within the dynamic context of\n"
+     "the throw.");
+
+  return scm_c_with_throw_handler (key,
+                                   scm_body_thunk, &c, 
+                                   scm_handle_by_proc, &handler, 0);
+}
+#undef FUNC_NAME
+
+
 
 void
 scm_i_init_deprecated ()
