@@ -205,9 +205,8 @@ scm_gc_malloc (size_t size, const char *what)
 void *
 scm_gc_calloc (size_t size, const char *what)
 {
-  void *ptr = scm_gc_malloc (size, what);
-  memset (ptr, 0x0, size);
-  return ptr;
+  /* `GC_MALLOC ()' always returns a zeroed buffer.  */
+  return scm_gc_malloc (size, what);
 }
 
 
@@ -323,13 +322,8 @@ scm_must_free (void *obj)
 #ifdef GUILE_DEBUG_MALLOC
   scm_malloc_unregister (obj);
 #endif
-  if (obj)
-    free (obj);
-  else
-    {
-      fprintf (stderr,"freeing NULL pointer");
-      abort ();
-    }
+
+  free (obj);
 }
 #undef FUNC_NAME
 
