@@ -30,6 +30,7 @@
 #include "libguile/chars.h"
 #include "libguile/continuations.h"
 #include "libguile/smob.h"
+#include "libguile/control.h"
 #include "libguile/eval.h"
 #include "libguile/macros.h"
 #include "libguile/procprop.h"
@@ -773,6 +774,12 @@ iprin1 (SCM exp, SCM port, scm_print_state *pstate)
 	case scm_tc7_vm_cont:
 	  scm_i_vm_cont_print (exp, port, pstate);
 	  break;
+	case scm_tc7_prompt:
+	  scm_i_prompt_print (exp, port, pstate);
+	  break;
+	case scm_tc7_with_fluids:
+	  scm_i_with_fluids_print (exp, port, pstate);
+	  break;
 	case scm_tc7_wvect:
 	  ENTER_NESTED_DATA (pstate, exp, circref);
 	  if (SCM_IS_WHVEC (exp))
