@@ -28,6 +28,7 @@
 #include <gc/gc_mark.h>
 
 #include "_scm.h"
+#include "control.h"
 #include "frames.h"
 #include "instructions.h"
 #include "objcodes.h"
@@ -79,74 +80,105 @@ scm_i_vm_cont_print (SCM x, SCM port, scm_print_state *pstate)
   scm_puts (">", port);
 }
 
-static SCM
-capture_vm_cont (struct scm_vm *vp)
-{
-  struct scm_vm_cont *p = scm_gc_malloc (sizeof (*p), "capture_vm_cont");
-  p->stack_size = vp->sp - vp->stack_base + 1;
-  p->stack_base = scm_gc_malloc (p->stack_size * sizeof (SCM),
-				 "capture_vm_cont");
-#ifdef VM_ENABLE_STACK_NULLING
-  if (vp->sp >= vp->stack_base)
-    if (!vp->sp[0] || vp->sp[1])
-      abort ();
-  memset (p->stack_base, 0, p->stack_size * sizeof (SCM));
-#endif
-  p->ip = vp->ip;
-  p->sp = vp->sp;
-  p->fp = vp->fp;
-  memcpy (p->stack_base, vp->stack_base, p->stack_size * sizeof (SCM));
-  p->reloc = p->stack_base - vp->stack_base;
-  return scm_cell (scm_tc7_vm_cont, (scm_t_bits)p);
-}
-
-static void
-reinstate_vm_cont (struct scm_vm *vp, SCM cont)
-{
-  struct scm_vm_cont *p = SCM_VM_CONT_DATA (cont);
-  if (vp->stack_size < p->stack_size)
-    {
-      /* puts ("FIXME: Need to expand"); */
-      abort ();
-    }
-#ifdef VM_ENABLE_STACK_NULLING
-  {
-    scm_t_ptrdiff nzero = (vp->sp - p->sp);
-    if (nzero > 0)
-      memset (vp->stack_base + p->stack_size, 0, nzero * sizeof (SCM));
-    /* actually nzero should always be negative, because vm_reset_stack will
-       unwind the stack to some point *below* this continuation */
-  }
-#endif
-  vp->ip = p->ip;
-  vp->sp = p->sp;
-  vp->fp = p->fp;
-  memcpy (vp->stack_base, p->stack_base, p->stack_size * sizeof (SCM));
-}
-
 /* In theory, a number of vm instances can be active in the call trace, and we
    only want to reify the continuations of those in the current continuation
    root. I don't see a nice way to do this -- ideally it would involve dynwinds,
    and previous values of the *the-vm* fluid within the current continuation
    root. But we don't have access to continuation roots in the dynwind stack.
-   So, just punt for now -- take the current value of *the-vm*.
+   So, just punt for now, we just capture the continuation for the current VM.
 
    While I'm on the topic, ideally we could avoid copying the C stack if the
    continuation root is inside VM code, and call/cc was invoked within that same
    call to vm_run; but that's currently not implemented.
  */
 SCM
-scm_vm_capture_continuations (void)
+scm_i_vm_capture_stack (SCM *stack_base, SCM *fp, SCM *sp, scm_t_uint8 *ra,
+                        scm_t_uint8 *mvra, scm_t_uint32 flags)
 {
-  SCM vm = scm_the_vm ();
-  return scm_acons (vm, capture_vm_cont (SCM_VM_DATA (vm)), SCM_EOL);
+  struct scm_vm_cont *p;
+
+  p = scm_gc_malloc (sizeof (*p), "capture_vm_cont");
+  p->stack_size = sp - stack_base + 1;
+  p->stack_base = scm_gc_malloc (p->stack_size * sizeof (SCM),
+				 "capture_vm_cont");
+#if defined(VM_ENABLE_STACK_NULLING) && 0
+  /* Tail continuations leave their frame on the stack for subsequent
+     application, but don't capture the frame -- so there are some elements on
+     the stack then, and this check doesn't work, so disable it for now. */
+  if (sp >= vp->stack_base)
+    if (!vp->sp[0] || vp->sp[1])
+      abort ();
+  memset (p->stack_base, 0, p->stack_size * sizeof (SCM));
+#endif
+  p->ra = ra;
+  p->mvra = mvra;
+  p->sp = sp;
+  p->fp = fp;
+  memcpy (p->stack_base, stack_base, (sp + 1 - stack_base) * sizeof (SCM));
+  p->reloc = p->stack_base - stack_base;
+  p->flags = flags;
+  return scm_cell (scm_tc7_vm_cont, (scm_t_bits)p);
 }
 
-void
-scm_vm_reinstate_continuations (SCM conts)
+static void
+vm_return_to_continuation (SCM vm, SCM cont, size_t n, SCM *argv)
 {
-  for (; conts != SCM_EOL; conts = SCM_CDR (conts))
-    reinstate_vm_cont (SCM_VM_DATA (SCM_CAAR (conts)), SCM_CDAR (conts));
+  struct scm_vm *vp;
+  struct scm_vm_cont *cp;
+  SCM *argv_copy;
+
+  argv_copy = alloca (n * sizeof(SCM));
+  memcpy (argv_copy, argv, n * sizeof(SCM));
+
+  vp = SCM_VM_DATA (vm);
+  cp = SCM_VM_CONT_DATA (cont);
+
+  if (n == 0 && !cp->mvra)
+    scm_misc_error (NULL, "Too few values returned to continuation",
+                    SCM_EOL);
+
+  if (vp->stack_size < cp->stack_size + n + 1)
+    scm_misc_error ("vm-engine", "not enough space to reinstate continuation",
+                    scm_list_2 (vm, cont));
+
+#ifdef VM_ENABLE_STACK_NULLING
+  {
+    scm_t_ptrdiff nzero = (vp->sp - cp->sp);
+    if (nzero > 0)
+      memset (vp->stack_base + cp->stack_size, 0, nzero * sizeof (SCM));
+    /* actually nzero should always be negative, because vm_reset_stack will
+       unwind the stack to some point *below* this continuation */
+  }
+#endif
+  vp->sp = cp->sp;
+  vp->fp = cp->fp;
+  memcpy (vp->stack_base, cp->stack_base, cp->stack_size * sizeof (SCM));
+
+  if (n == 1 || !cp->mvra)
+    {
+      vp->ip = cp->ra;
+      vp->sp++;
+      *vp->sp = argv_copy[0];
+    }
+  else
+    {
+      size_t i;
+      for (i = 0; i < n; i++)
+        {
+          vp->sp++;
+          *vp->sp = argv_copy[i];
+        }
+      vp->sp++;
+      *vp->sp = scm_from_size_t (n);
+      vp->ip = cp->mvra;
+    }
+}
+
+SCM
+scm_i_vm_capture_continuation (SCM vm)
+{
+  struct scm_vm *vp = SCM_VM_DATA (vm);
+  return scm_i_vm_capture_stack (vp->stack_base, vp->fp, vp->sp, vp->ip, NULL, 0);
 }
 
 static void
@@ -169,15 +201,117 @@ vm_dispatch_hook (SCM vm, int hook_num)
   vp->trace_level++;
 }
 
+static void vm_abort (SCM vm, size_t n, scm_t_int64 cookie) SCM_NORETURN;
+static void
+vm_abort (SCM vm, size_t n, scm_t_int64 vm_cookie)
+{
+  size_t i;
+  ssize_t tail_len;
+  SCM tag, tail, *argv;
+  
+  /* FIXME: VM_ENABLE_STACK_NULLING */
+  tail = *(SCM_VM_DATA (vm)->sp--);
+  /* NULLSTACK (1) */
+  tail_len = scm_ilength (tail);
+  if (tail_len < 0)
+    scm_misc_error ("vm-engine", "tail values to abort should be a list",
+                    scm_list_1 (tail));
+
+  tag = SCM_VM_DATA (vm)->sp[-n];
+  argv = alloca ((n + tail_len) * sizeof (SCM));
+  for (i = 0; i < n; i++)
+    argv[i] = SCM_VM_DATA (vm)->sp[-(n-1-i)];
+  for (; i < n + tail_len; i++, tail = scm_cdr (tail))
+    argv[i] = scm_car (tail);
+  /* NULLSTACK (n + 1) */
+  SCM_VM_DATA (vm)->sp -= n + 1;
+
+  scm_c_abort (vm, tag, n + tail_len, argv, vm_cookie);
+}
+
+static void
+vm_reinstate_partial_continuation (SCM vm, SCM cont, SCM intwinds,
+                                   size_t n, SCM *argv, scm_t_int64 vm_cookie)
+{
+  struct scm_vm *vp;
+  struct scm_vm_cont *cp;
+  SCM *argv_copy, *base;
+  size_t i;
+
+  argv_copy = alloca (n * sizeof(SCM));
+  memcpy (argv_copy, argv, n * sizeof(SCM));
+
+  vp = SCM_VM_DATA (vm);
+  cp = SCM_VM_CONT_DATA (cont);
+  base = SCM_FRAME_UPPER_ADDRESS (vp->fp) + 1;
+
+#define RELOC(scm_p) (scm_p + cp->reloc + (base - cp->stack_base))
+
+  if ((base - vp->stack_base) + cp->stack_size + n + 1 > vp->stack_size)
+    scm_misc_error ("vm-engine",
+                    "not enough space to instate partial continuation",
+                    scm_list_2 (vm, cont));
+
+  memcpy (base, cp->stack_base, cp->stack_size * sizeof (SCM));
+
+  /* now relocate frame pointers */
+  {
+    SCM *fp;
+    for (fp = RELOC (cp->fp);
+         SCM_FRAME_LOWER_ADDRESS (fp) > base;
+         fp = SCM_FRAME_DYNAMIC_LINK (fp))
+      SCM_FRAME_SET_DYNAMIC_LINK (fp, RELOC (SCM_FRAME_DYNAMIC_LINK (fp)));
+  }
+
+  vp->sp = base - 1 + cp->stack_size;
+  vp->fp = RELOC (cp->fp);
+  vp->ip = cp->mvra;
+
+  /* now push args. ip is in a MV context. */
+  for (i = 0; i < n; i++)
+    {
+      vp->sp++;
+      *vp->sp = argv_copy[i];
+    }
+  vp->sp++;
+  *vp->sp = scm_from_size_t (n);
+
+  /* Finally, rewind the dynamic state.
+
+     We have to treat prompts specially, because we could be rewinding the
+     dynamic state from a different thread, or just a different position on the
+     C and/or VM stack -- so we need to reset the jump buffers so that an abort
+     comes back here, with appropriately adjusted sp and fp registers. */
+  {
+    long delta = 0;
+    SCM newwinds = scm_i_dynwinds ();
+    for (; scm_is_pair (intwinds); intwinds = scm_cdr (intwinds), delta--)
+      {
+        SCM x = scm_car (intwinds);
+        if (SCM_PROMPT_P (x))
+          /* the jmpbuf will be reset by our caller */
+          x = scm_c_make_prompt (SCM_PROMPT_TAG (x),
+                                 RELOC (SCM_PROMPT_REGISTERS (x)->fp),
+                                 RELOC (SCM_PROMPT_REGISTERS (x)->sp),
+                                 SCM_PROMPT_REGISTERS (x)->ip,
+                                 SCM_PROMPT_ESCAPE_P (x),
+                                 vm_cookie,
+                                 newwinds);
+        newwinds = scm_cons (x, newwinds);
+      }
+    scm_dowinds (newwinds, delta);
+  }
+#undef RELOC
+}
+
 
 /*
  * VM Internal functions
  */
 
-SCM_SYMBOL (sym_vm_run, "vm-run");
-SCM_SYMBOL (sym_vm_error, "vm-error");
-SCM_SYMBOL (sym_keyword_argument_error, "keyword-argument-error");
-SCM_SYMBOL (sym_debug, "debug");
+/* Unfortunately we can't snarf these: snarfed things are only loaded up from
+   (system vm vm), which might not be loaded before an error happens. */
+static SCM sym_vm_run, sym_vm_error, sym_keyword_argument_error, sym_debug;
 
 void
 scm_i_vm_print (SCM x, SCM port, scm_print_state *pstate)
@@ -197,7 +331,9 @@ really_make_boot_program (long nargs)
   SCM ret;
 
   if (SCM_UNLIKELY (nargs > 255 || nargs < 0))
-    abort ();
+    scm_misc_error ("vm-engine", "too many args when making boot procedure",
+                    scm_list_1 (scm_from_long (nargs)));
+
   text[1] = (scm_t_uint8)nargs;
 
   bp = scm_malloc (sizeof (struct scm_objcode) + sizeof (text));
@@ -338,6 +474,7 @@ make_vm (void)
   vp->trace_level = 0;
   for (i = 0; i < SCM_VM_NUM_HOOKS; i++)
     vp->hooks[i] = SCM_BOOL_F;
+  vp->cookie = 0;
   return scm_cell (scm_tc7_vm, (scm_t_bits)vp);
 }
 #undef FUNC_NAME
@@ -407,12 +544,6 @@ SCM_DEFINE (scm_vm_apply, "vm-apply", 3, 0, 0,
 }
 #undef FUNC_NAME
 
-SCM
-scm_vm_call_with_new_stack (SCM vm, SCM thunk, SCM id)
-{
-  return scm_c_vm_run (vm, thunk, NULL, 0);
-}
-
 /* Scheme interface */
 
 SCM_DEFINE (scm_vm_version, "vm-version", 0, 0, 0,
@@ -628,9 +759,15 @@ SCM scm_load_compiled_with_vm (SCM file)
 void
 scm_bootstrap_vm (void)
 {
-  scm_c_register_extension ("libguile", "scm_init_vm",
+  scm_c_register_extension ("libguile-" SCM_EFFECTIVE_VERSION,
+                            "scm_init_vm",
                             (scm_t_extension_init_func)scm_init_vm, NULL);
 
+  sym_vm_run = scm_from_locale_symbol ("vm-run");
+  sym_vm_error = scm_from_locale_symbol ("vm-error");
+  sym_keyword_argument_error = scm_from_locale_symbol ("keyword-argument-error");
+  sym_debug = scm_from_locale_symbol ("debug");
+
 #ifdef VM_ENABLE_PRECISE_STACK_GC_SCAN
   vm_stack_gc_kind =
     GC_new_kind (GC_new_free_list (),
