@@ -22,6 +22,7 @@
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-9)
   #:use-module (srfi srfi-11)
+  #:use-module (ice-9 vlist)
   #:use-module (system base syntax)
   #:use-module (system base message)
   #:use-module (system vm program)
@@ -81,9 +82,9 @@
 ;; this specific case. A proper solution would be some sort of liveness
 ;; analysis, and not our linear allocation algorithm.
 ;;
-;; Closure variables are captured when a closure is created, and stored
-;; in a vector. Each closure variable has a unique index into that
-;; vector.
+;; Closure variables are captured when a closure is created, and stored in a
+;; vector inline to the closure object itself. Each closure variable has a
+;; unique index into that vector.
 ;;
 ;; There is one more complication. Procedures bound by <fix> may, in
 ;; some cases, be rendered inline to their parent procedure. That is to
@@ -124,11 +125,18 @@
 ;; generated code can skip argument checks at runtime if they match at
 ;; compile-time.
 ;;
+;; Also, while we're a-traversing and an-allocating, we check prompt
+;; handlers to see if the "continuation" argument is used. If not, we
+;; mark the prompt as being "escape-only". This allows us to implement
+;; `catch' and `throw' using `prompt' and `control', but without causing
+;; a continuation to be reified. Heh heh.
+;;
 ;; That is:
 ;;
 ;;  sym -> {lambda -> address}
 ;;  lambda -> (labels . free-locs)
 ;;  lambda-case -> (gensym . nlocs)
+;;  prompt -> escape-only?
 ;;
 ;; address ::= (local? boxed? . index)
 ;; labels ::= ((sym . lambda) ...)
@@ -328,6 +336,24 @@
       ((<let-values> exp body)
        (lset-union eq? (step exp) (step body)))
       
+      ((<dynwind> body winder unwinder)
+       (lset-union eq? (step body) (step winder) (step unwinder)))
+      
+      ((<dynlet> fluids vals body)
+       (apply lset-union eq? (step body) (map step (append fluids vals))))
+      
+      ((<dynref> fluid)
+       (step fluid))
+      
+      ((<dynset> fluid exp)
+       (lset-union eq? (step fluid) (step exp)))
+      
+      ((<prompt> tag body handler)
+       (lset-union eq? (step tag) (step body) (step handler)))
+      
+      ((<abort> tag args tail)
+       (apply lset-union eq? (step tag) (step tail) (map step args)))
+      
       (else '())))
   
   ;; allocation: sym -> {lambda -> address}
@@ -479,6 +505,29 @@
       ((<let-values> exp body)
        (max (recur exp) (recur body)))
       
+      ((<dynwind> body winder unwinder)
+       (max (recur body) (recur winder) (recur unwinder)))
+      
+      ((<dynlet> fluids vals body)
+       (apply max (recur body) (map recur (append fluids vals))))
+      
+      ((<dynref> fluid)
+       (recur fluid))
+      
+      ((<dynset> fluid exp)
+       (max (recur fluid) (recur exp)))
+      
+      ((<prompt> tag body handler)
+       (let ((cont-var (and (lambda-case? handler)
+                            (pair? (lambda-case-vars handler))
+                            (car (lambda-case-vars handler)))))
+         (hashq-set! allocation x
+                     (and cont-var (zero? (hashq-ref refcounts cont-var 0))))
+         (max (recur tag) (recur body) (recur handler))))
+      
+      ((<abort> tag args tail)
+       (apply max (recur tag) (recur tail) (map recur args)))
+      
       (else n)))
 
   (analyze! x #f '() #t #f)
@@ -561,7 +610,7 @@ accurate information is missing from a given `tree-il' element."
            (vars (binding-info-vars info)))
        (record-case x
          ((<lexical-ref> gensym)
-          (make-binding-info vars (cons gensym refs)))
+          (make-binding-info vars (vhash-consq gensym #t refs)))
          (else info))))
 
    (lambda (x info env locs)
@@ -571,14 +620,15 @@ accurate information is missing from a given `tree-il' element."
            (vars (binding-info-vars info))
            (src  (tree-il-src x)))
        (define (extend inner-vars inner-names)
-         (append (map (lambda (var name)
-                        (list var name src))
-                      inner-vars
-                      inner-names)
-                 vars))
+         (fold (lambda (var name vars)
+                 (vhash-consq var (list name src) vars))
+               vars
+               inner-vars
+               inner-names))
+
        (record-case x
          ((<lexical-set> gensym)
-          (make-binding-info vars (cons gensym refs)))
+          (make-binding-info vars (vhash-consq gensym #t refs)))
          ((<lambda-case> req opt inits rest kw vars)
           (let ((names `(,@req
                          ,@(or opt '())
@@ -599,25 +649,21 @@ accurate information is missing from a given `tree-il' element."
      (let ((refs (binding-info-refs info))
            (vars (binding-info-vars info)))
        (define (shrink inner-vars refs)
-         (for-each (lambda (var)
-                     (let ((gensym (car var)))
-                       ;; Don't report lambda parameters as
-                       ;; unused.
-                       (if (and (not (memq gensym refs))
-                                (not (and (lambda-case? x)
-                                          (memq gensym
-                                                inner-vars))))
-                           (let ((name (cadr var))
-                                 ;; We can get approximate
-                                 ;; source location by going up
-                                 ;; the LOCS location stack.
-                                 (loc  (or (caddr var)
-                                           (find pair? locs))))
-                             (warning 'unused-variable loc name)))))
-                   (filter (lambda (var)
-                             (memq (car var) inner-vars))
-                           vars))
-         (fold alist-delete vars inner-vars))
+         (vlist-for-each
+          (lambda (var)
+            (let ((gensym (car var)))
+              ;; Don't report lambda parameters as unused.
+              (if (and (memq gensym inner-vars)
+                       (not (vhash-assq gensym refs))
+                       (not (lambda-case? x)))
+                  (let ((name (cadr var))
+                        ;; We can get approximate source location by going up
+                        ;; the LOCS location stack.
+                        (loc  (or (caddr var)
+                                  (find pair? locs))))
+                    (warning 'unused-variable loc name)))))
+          vars)
+         (vlist-drop vars (length inner-vars)))
 
        ;; For simplicity, we leave REFS untouched, i.e., with
        ;; names of variables that are now going out of scope.
@@ -635,7 +681,7 @@ accurate information is missing from a given `tree-il' element."
          (else info))))
 
    (lambda (result env) #t)
-   (make-binding-info '() '())))
+   (make-binding-info vlist-null vlist-null)))
 
 
 ;;;
@@ -654,9 +700,10 @@ accurate information is missing from a given `tree-il' element."
   (refs             reference-graph-refs) ;; ((REF-CONTEXT REF ...) ...)
   (toplevel-context reference-graph-toplevel-context)) ;; NAME | #f
 
-(define (graph-reachable-nodes root refs)
-  ;; Return the list of nodes reachable from ROOT in graph REFS.  REFS is an alist
-  ;; representing edges: ((A B C) (B A) (C)) corresponds to
+(define (graph-reachable-nodes root refs reachable)
+  ;; Add to REACHABLE the nodes reachable from ROOT in graph REFS.  REFS is a
+  ;; vhash mapping nodes to the list of their children: for instance,
+  ;; ((A -> (B C)) (B -> (A)) (C -> ())) corresponds to
   ;;
   ;;  ,-------.
   ;;  v       |
@@ -664,30 +711,49 @@ accurate information is missing from a given `tree-il' element."
   ;;  |
   ;;  v
   ;;  C
+  ;;
+  ;; REACHABLE is a vhash of nodes known to be otherwise reachable.
 
   (let loop ((root   root)
-             (path   '())
-             (result '()))
-    (if (or (memq root path)
-            (memq root result))
+             (path   vlist-null)
+             (result reachable))
+    (if (or (vhash-assq root path)
+            (vhash-assq root result))
         result
-        (let ((children (assoc-ref refs root)))
-          (if (not children)
-              result
-              (let ((path (cons root path)))
-                (append children
-                        (fold (lambda (child result)
-                                (loop child path result))
-                              result
-                              children))))))))
+        (let* ((children (or (and=> (vhash-assq root refs) cdr) '()))
+               (path     (vhash-consq root #t path))
+               (result   (fold (lambda (kid result)
+                                 (loop kid path result))
+                               result
+                               children)))
+          (fold (lambda (kid result)
+                  (vhash-consq kid #t result))
+                result
+                children)))))
 
 (define (graph-reachable-nodes* roots refs)
   ;; Return the list of nodes in REFS reachable from the nodes listed in ROOTS.
-  ;; FIXME: Choose a more efficient algorithm.
-  (apply lset-union eq?
-         (map (lambda (node)
-                (cons node (graph-reachable-nodes node refs)))
-              roots)))
+  (vlist-fold (lambda (root+true result)
+                (let* ((root      (car root+true))
+                       (reachable (graph-reachable-nodes root refs result)))
+                  (vhash-consq root #t reachable)))
+              vlist-null
+              roots))
+
+(define (partition* pred vhash)
+  ;; Partition VHASH according to PRED.  Return the two resulting vhashes.
+  (let ((result
+         (vlist-fold (lambda (k+v result)
+                       (let ((k  (car k+v))
+                             (v  (cdr k+v))
+                             (r1 (car result))
+                             (r2 (cdr result)))
+                         (if (pred k)
+                             (cons (vhash-consq k v r1) r2)
+                             (cons r1 (vhash-consq k v r2)))))
+                     (cons vlist-null vlist-null)
+                     vhash)))
+    (values (car result) (cdr result))))
 
 (define unused-toplevel-analysis
   ;; Report unused top-level definitions that are not exported.
@@ -697,9 +763,8 @@ accurate information is missing from a given `tree-il' element."
            (let* ((refs     (reference-graph-refs graph))
                   (defs     (reference-graph-defs graph))
                   (ctx      (reference-graph-toplevel-context graph))
-                  (ctx-refs (or (assoc-ref refs ctx) '())))
-             (make-reference-graph (alist-cons ctx (cons name ctx-refs)
-                                               (alist-delete ctx refs eq?))
+                  (ctx-refs (or (and=> (vhash-assq ctx refs) cdr) '())))
+             (make-reference-graph (vhash-consq ctx (cons name ctx-refs) refs)
                                    defs ctx)))))
     (define (macro-variable? name env)
       (and (module? env)
@@ -724,8 +789,8 @@ accurate information is missing from a given `tree-il' element."
          (record-case x
            ((<toplevel-define> name src)
             (let ((refs refs)
-                  (defs (alist-cons name (or src (find pair? locs))
-                                    defs)))
+                  (defs (vhash-consq name (or src (find pair? locs))
+                                     defs)))
               (make-reference-graph refs defs name)))
            ((<toplevel-set> name src)
             (add-ref-from-context graph name))
@@ -755,27 +820,24 @@ accurate information is missing from a given `tree-il' element."
              #t))
 
        (let-values (((public-defs private-defs)
-                     (partition (lambda (name+src)
-                                  (let ((name (car name+src)))
-                                    (or (exported? name)
-                                        (macro-variable? name env))))
-                                (reference-graph-defs graph))))
-         (let* ((roots     (cons #f (map car public-defs)))
+                     (partition* (lambda (name)
+                                   (or (exported? name)
+                                       (macro-variable? name env)))
+                                 (reference-graph-defs graph))))
+         (let* ((roots     (vhash-consq #f #t public-defs))
                 (refs      (reference-graph-refs graph))
                 (reachable (graph-reachable-nodes* roots refs))
-                (unused    (filter (lambda (name+src)
-                                     ;; FIXME: This is inefficient when
-                                     ;; REACHABLE is large (e.g., boot-9.scm);
-                                     ;; use a vhash or equivalent.
-                                     (not (memq (car name+src) reachable)))
-                                   private-defs)))
-           (for-each (lambda (name+loc)
-                       (let ((name (car name+loc))
-                             (loc  (cdr name+loc)))
-                         (warning 'unused-toplevel loc name)))
-                     (reverse unused)))))
+                (unused    (vlist-filter (lambda (name+src)
+                                           (not (vhash-assq (car name+src)
+                                                            reachable)))
+                                         private-defs)))
+           (vlist-for-each (lambda (name+loc)
+                             (let ((name (car name+loc))
+                                   (loc  (cdr name+loc)))
+                               (warning 'unused-toplevel loc name)))
+                           unused))))
 
-     (make-reference-graph '() '() #f))))
+     (make-reference-graph vlist-null vlist-null #f))))
 
 
 ;;;
@@ -827,14 +889,14 @@ accurate information is missing from a given `tree-il' element."
        (define (bound? name)
          (or (and (module? env)
                   (module-variable env name))
-             (memq name defs)))
+             (vhash-assq name defs)))
 
        (record-case x
          ((<toplevel-ref> name src)
           (if (bound? name)
               info
               (let ((src (or src (find pair? locs))))
-                (make-toplevel-info (alist-cons name src refs)
+                (make-toplevel-info (vhash-consq name src refs)
                                     defs))))
          (else info))))
 
@@ -846,18 +908,18 @@ accurate information is missing from a given `tree-il' element."
        (define (bound? name)
          (or (and (module? env)
                   (module-variable env name))
-             (memq name defs)))
+             (vhash-assq name defs)))
 
        (record-case x
          ((<toplevel-set> name src)
           (if (bound? name)
               (make-toplevel-info refs defs)
               (let ((src (find pair? locs)))
-                (make-toplevel-info (alist-cons name src refs)
+                (make-toplevel-info (vhash-consq name src refs)
                                     defs))))
          ((<toplevel-define> name)
-          (make-toplevel-info (alist-delete name refs eq?)
-                              (cons name defs)))
+          (make-toplevel-info (vhash-delete name refs eq?)
+                              (vhash-consq name #t defs)))
 
          ((<application> proc args)
           ;; Check for a dynamic top-level definition, as is
@@ -865,9 +927,9 @@ accurate information is missing from a given `tree-il' element."
           (let ((name (goops-toplevel-definition proc args
                                                  env)))
             (if (symbol? name)
-                (make-toplevel-info (alist-delete name refs
+                (make-toplevel-info (vhash-delete name refs
                                                   eq?)
-                                    (cons name defs))
+                                    (vhash-consq name #t defs))
                 (make-toplevel-info refs defs))))
          (else
           (make-toplevel-info refs defs)))))
@@ -878,13 +940,13 @@ accurate information is missing from a given `tree-il' element."
 
    (lambda (toplevel env)
      ;; Post-process the result.
-     (for-each (lambda (name+loc)
-                 (let ((name (car name+loc))
-                       (loc  (cdr name+loc)))
-                   (warning 'unbound-variable loc name)))
-               (reverse (toplevel-info-refs toplevel))))
+     (vlist-for-each (lambda (name+loc)
+                       (let ((name (car name+loc))
+                             (loc  (cdr name+loc)))
+                         (warning 'unbound-variable loc name)))
+                     (vlist-reverse (toplevel-info-refs toplevel))))
 
-   (make-toplevel-info '() '())))
+   (make-toplevel-info vlist-null vlist-null)))
 
 
 ;;;
@@ -1005,20 +1067,20 @@ accurate information is missing from a given `tree-il' element."
          (record-case val
            ((<lambda> body)
             (make-arity-info toplevel-calls
-                             (alist-cons lexical-name val
-                                         lexical-lambdas)
+                             (vhash-consq lexical-name val
+                                          lexical-lambdas)
                              toplevel-lambdas))
            ((<lexical-ref> gensym)
             ;; lexical alias
-            (let ((val* (assq gensym lexical-lambdas)))
+            (let ((val* (vhash-assq gensym lexical-lambdas)))
               (if (pair? val*)
                   (extend lexical-name (cdr val*) info)
                   info)))
            ((<toplevel-ref> name)
             ;; top-level alias
             (make-arity-info toplevel-calls
-                             (alist-cons lexical-name val
-                                         lexical-lambdas)
+                             (vhash-consq lexical-name val
+                                          lexical-lambdas)
                              toplevel-lambdas))
            (else info))))
 
@@ -1032,17 +1094,17 @@ accurate information is missing from a given `tree-il' element."
             ((<lambda> body)
              (make-arity-info toplevel-calls
                               lexical-lambdas
-                              (alist-cons name exp toplevel-lambdas)))
+                              (vhash-consq name exp toplevel-lambdas)))
             ((<toplevel-ref> name)
              ;; alias for another toplevel
-             (let ((proc (assq name toplevel-lambdas)))
+             (let ((proc (vhash-assq name toplevel-lambdas)))
                (make-arity-info toplevel-calls
                                 lexical-lambdas
-                                (alist-cons (toplevel-define-name x)
-                                            (if (pair? proc)
-                                                (cdr proc)
-                                                exp)
-                                            toplevel-lambdas))))
+                                (vhash-consq (toplevel-define-name x)
+                                             (if (pair? proc)
+                                                 (cdr proc)
+                                                 exp)
+                                             toplevel-lambdas))))
             (else info)))
          ((<let> vars vals)
           (fold extend info vars vals))
@@ -1057,16 +1119,16 @@ accurate information is missing from a given `tree-il' element."
              (validate-arity proc x #t)
              info)
             ((<toplevel-ref> name)
-             (make-arity-info (alist-cons name x toplevel-calls)
+             (make-arity-info (vhash-consq name x toplevel-calls)
                               lexical-lambdas
                               toplevel-lambdas))
             ((<lexical-ref> gensym)
-             (let ((proc (assq gensym lexical-lambdas)))
+             (let ((proc (vhash-assq gensym lexical-lambdas)))
                (if (pair? proc)
                    (record-case (cdr proc)
                      ((<toplevel-ref> name)
                       ;; alias to toplevel
-                      (make-arity-info (alist-cons name x toplevel-calls)
+                      (make-arity-info (vhash-consq name x toplevel-calls)
                                        lexical-lambdas
                                        toplevel-lambdas))
                      (else
@@ -1087,7 +1149,9 @@ accurate information is missing from a given `tree-il' element."
              (lexical-lambdas  (lexical-lambdas info))
              (toplevel-lambdas (toplevel-lambdas info)))
          (make-arity-info toplevel-calls
-                          (alist-delete name lexical-lambdas eq?)
+                          (if (vhash-assq name lexical-lambdas)
+                              (vlist-tail lexical-lambdas)
+                              lexical-lambdas)
                           toplevel-lambdas)))
 
      (let ((toplevel-calls   (toplevel-procedure-calls info))
@@ -1108,26 +1172,25 @@ accurate information is missing from a given `tree-il' element."
      ;; encountered.
      (let ((toplevel-calls   (toplevel-procedure-calls result))
            (toplevel-lambdas (toplevel-lambdas result)))
-       (for-each (lambda (name+application)
-                   (let* ((name        (car name+application))
-                          (application (cdr name+application))
-                          (proc
-                           (or (assoc-ref toplevel-lambdas name)
-                               (and (module? env)
-                                    (false-if-exception
-                                     (module-ref env name)))))
-                          (proc*
-                           ;; handle toplevel aliases
-                           (if (toplevel-ref? proc)
-                               (let ((name (toplevel-ref-name proc)))
-                                 (and (module? env)
-                                      (false-if-exception
-                                       (module-ref env name))))
-                               proc)))
-                     ;; (format #t "toplevel-call to ~A (~A) from ~A~%"
-                     ;;         name proc* application)
-                     (if (or (lambda? proc*) (procedure? proc*))
-                         (validate-arity proc* application (lambda? proc*)))))
-                 toplevel-calls)))
+       (vlist-for-each
+        (lambda (name+application)
+          (let* ((name        (car name+application))
+                 (application (cdr name+application))
+                 (proc
+                  (or (and=> (vhash-assq name toplevel-lambdas) cdr)
+                      (and (module? env)
+                           (false-if-exception
+                            (module-ref env name)))))
+                 (proc*
+                  ;; handle toplevel aliases
+                  (if (toplevel-ref? proc)
+                      (let ((name (toplevel-ref-name proc)))
+                        (and (module? env)
+                             (false-if-exception
+                              (module-ref env name))))
+                      proc)))
+            (if (or (lambda? proc*) (procedure? proc*))
+                (validate-arity proc* application (lambda? proc*)))))
+        toplevel-calls)))
 
-   (make-arity-info '() '() '())))
+   (make-arity-info vlist-null vlist-null vlist-null)))
